from pathlib import Path

JOBS_PATH = Path.cwd() / "jobs"
MODULES_PATH = Path.cwd() / "modules"
NOTEBOOKS_PATH = Path.cwd() / "notebooks"
TOOLS_PATH = Path.cwd() / "tools"
TESTS_PATH = Path.cwd() / "tests"
